package Common;

use Exporter;
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK);
@EXPORT = qw(random_string replace EMERGENCY CRISIS CRITICAL ERROR WARNING NORMAL INFO DEBUG);

@ISA = qw(Exporter);
@EXPORT_OK = @EXPORT;

=head1 NAME

Common

=cut

=head1 SYNOPSIS

use Common qw(random_string replace);

=cut

=head1 DESCRIPTION

Some common functions used throughout the code, regardless of module.

=cut

=head1 FUNCTIONS

=head2 random_string(@array)

Returns a random array element.

=cut

sub random_string
{
    return $_[rand($#_ + 1)];
}

=head2 replace($scalar, $hashref)

For every key in the hash, every instance of '$KEY' in $scalar will be replaced.

Example:

replace('Some $STRING', { STRING => $scalar })

=cut

sub replace
{
    my $string = shift;
    my $hashref = shift;
    
    return "Invalid use of replace()" unless($string and ref($hashref) eq 'HASH');
    
    foreach my $key (keys %{$hashref}) {
       $string =~ s/\$$key/$hashref->{$key}/g;
    }
    
    return $string;
}

#A few constants. (For logging)
use constant EMERGENCY => 0;
use constant CRISIS => 1;
use constant CRITICAL => 2;
use constant ERROR => 3;
use constant WARNING => 4;
use constant NORMAL => 5;
use constant INFO => 6;
use constant DEBUG => 7;

1;
