#Adapted from infobot(.org) to Erwin.
# Original headers:
# infobot copyright (C) kevin lenzo 1997-99
# He used the free "Artistic License".

package Calc;
use strict;

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
bindtextdomain('erwin', '/usr/share/locale/');
textdomain('erwin');

sub Calc {
  my ($core, $backrefs) = @_;
  return if($core->testing);
  my $message = join("", @$backrefs);

  if (($message !~ /^\s*$/) and ($message !~ /(\d+\.){2,}/)) {
    my($locMsg) = $message;
    my ($exp, $val);

    while ($locMsg =~ /(exp ([\w\d]+))/) {
      $exp = $1;
      $val = exp($2); 
      $locMsg =~ s/$exp/+$val/g;
    }
    while ($locMsg =~ /(hex2dec\s*([0-9A-Fa-f]+))/) {
      $exp = $1;
      $val = hex($2);
      $locMsg =~ s/$exp/+$val/g;
    }
    if ($locMsg =~ /^\s*(dec2hex\s*(\d+))\s*\?*/) {
      $exp = $1;
      $val = sprintf("%x", "$2");
      $locMsg =~ s/$exp/+$val/g;
    }

    my $e = exp(1);
    $locMsg =~ s/\be\b/$e/;

    while ($locMsg =~ /(log\s*((\d+\.?\d*)|\d*\.?\d+))\s*/) {
      $exp = $1;
      my $res = $2;
      if ($res == 0) { $val = "Infinity";} 
      else { $val = log($res); } ;
      $locMsg =~ s/$exp/+$val/g;
    }

    while ($locMsg =~ /(bin2dec ([01]+))/) {
      $exp = $1;
      $val = join ('', unpack ("B*", $2)) ;
      $locMsg =~ s/$exp/+$val/g;
    }

    while ($locMsg =~ /(dec2bin (\d+))/) {
      $exp = $1;
      $val = join('', unpack('B*', pack('N', $2)));
      $val =~ s/^0+//;
      $locMsg =~ s/$exp/+$val/g;
    }

    my $pi = 3.14159265359;
    $locMsg =~ s/pi/$pi/g;
    $locMsg =~ s/ to the / ** /g;
    $locMsg =~ s/\^/ ** /g;
    $locMsg =~ s/\btimes\b/\*/g;
    $locMsg =~ s/\bdiv(ided by)? /\/ /g;
    $locMsg =~ s/\bover /\/ /g;
    $locMsg =~ s/\bsquared/\*\*2 /g;
    $locMsg =~ s/\bcubed/\*\*3 /g;
    $locMsg =~ s/\bto\s+(\d+)(r?st|nd|rd|th)?( power)?/\*\*$1 /ig;
    $locMsg =~ s/\bpercent of/*0.01*/ig;
    $locMsg =~ s/\bpercent/*0.01/ig;
    $locMsg =~ s/\% of\b/*0.01*/g;
    $locMsg =~ s/\%/*0.01/g;
    $locMsg =~ s/\bsquare root of (\d+)/$1 ** 0.5 /ig;
    $locMsg =~ s/\bcubed? root of (\d+)/$1 **(1.0\/3.0) /ig;
    $locMsg =~ s/ of / * /;
    $locMsg =~ s/(bit(-| )?)?xor(\'?e?d( with))?/\^/g;
    $locMsg =~ s/(bit(-| )?)?or(\'?e?d( with))?/\|/g;
    $locMsg =~ s/bit(-| )?and(\'?e?d( with))?/\& /g;
    $locMsg =~ s/(plus|and)/+/ig;

    if (($locMsg =~ /^\s*[-\d*+\s()\/^\.\|\&\*\!]+\s*$/) 
	&& ($locMsg !~ /^\s*\(?\d+\.?\d*\)?\s*$/)
	&& ($locMsg !~ /^\s*$/)
	&& ($locMsg !~ /^\s*[( )]+\s*$/))
    {
      # $tmpMsg = $locMsg;

      $locMsg = eval($locMsg);

      if ($locMsg =~ /^[-+\de\.]+$/) {
	$locMsg = sprintf("%1.12f", $locMsg);
	$locMsg =~ s/\.?0+$//;
	$locMsg =~ s/(\.\d+)000\d+/$1/; 
	if (length($locMsg) > 80) {
	  $locMsg = gettext('A number with more than 80 digits.');
	}

        $core->isInfo;
	return $locMsg;
      }
    }
  }

  return undef;
}

1;
