package Facts;
use strict;
#use vars qw(%stopwords);

use Common;
use Digest::MD5 qw(md5_base64);
use My::DBI qw(:all);
use GDBM_File;

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
bindtextdomain('erwin', '/usr/share/locale/');
textdomain('erwin');

#For internal use. Retrieve and dereference a fact.
sub get_fact
{
   my($core, $keyword) = @_;
   return unless($keyword);

   $core->log(replace(gettext('Looking up "$KEYWORD"'), { KEYWORD => $keyword }), DEBUG);

   my %config;
   my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
   $core->module_config(\%config, $where);
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';

   $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;

   my %fact = dbi_exec_for_row_hash( $dbh, "SELECT * FROM $config{table} WHERE keyword=?", $keyword);

   if(!$fact{fact} && $fact{pointer}) {
       $core->log(replace(gettext('Dereferencing $KEYWORD to $POINTER'), { KEYWORD => $keyword, POINTER => $fact{pointer} }), DEBUG);
       %fact = dbi_exec_for_row_hash( $dbh, "SELECT * FROM $config{table} WHERE keyword=?", $fact{pointer});
       $keyword = $fact{keyword} if($fact{fact});
   }

   if($fact{fact}) {
      return ($keyword, \%fact);
   } else {
      return ($keyword, undef);
   }
}

#Lookup a factoid and return it. Dereference any pointers.
sub Lookup {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);

    my($keyword, $_who) = @$backrefs[0, 1];

    $keyword = lc($keyword);
   
    my $fact;

    $core->heap->{redirect} = $_who if($_who);
    $core->heap->{keyword} = $keyword;

    ($keyword, $fact) = get_fact($core, $keyword);

    $fact->{fact} =~ s/\$KEY/$keyword/g;
    
    $core->log('Lookup(): ' . replace(gettext('$KEYWORD looked up by $WHO'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}}), INFO);

    return $fact->{fact};
}

#Lookup a factoid and return it, but don't replace special variables.
sub Literal
{
    my($core, $backrefs, $args) = @_;
    return if($core->testing);
    $core->heap->{literal} = 1;
    $core->heap->{floodoverride} = 1;

    my $keyword = lc($backrefs->[0]);

   my %config;
   my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
   $core->module_config(\%config, $where);
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';
   $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;

    my %fact = dbi_exec_for_row_hash( $dbh, "SELECT * FROM $config{table} WHERE keyword=?", $keyword);

    return unless($fact{pointer} || $fact{fact});

    $core->log('Literal(): ' . replace(gettext('$KEYWORD looked up literally by $WHO'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}}), INFO);
    if($fact{pointer}) {
       return replace(gettext('$KEYWORD -> $POINTER'), { KEYWORD => $keyword, POINTER => $fact{pointer} });
    } else {
          return $fact{fact};
    }

}

sub Learn {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);
    my($keyword, $verb, $fact) = @$backrefs[0 .. 2];

    $keyword = lc($keyword);
    $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
    $keyword = $1;

   my %config;
   my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
   $core->module_config(\%config, $where);

   if(length($keyword) > $config{keyword_length}) {
      $core->isInfo;
      return replace(gettext('Your keyword is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($keyword), MAX => $config{keyword_length} });
   }
 
    #return replace(gettext('"$KEYWORD" is a stopword.'), { KEYWORD => $keyword }) if($stopwords{$keyword});

    #This implies that adding to a pointer really adds it to the original.
    my $oldfact; 
    ($keyword, $oldfact) = get_fact($core, $keyword);

    if ($oldfact) {
       #Append the new fact if one already exists.

       #Remove any whitespace or punctation and add new ones.
       $fact = "$oldfact->{fact} $keyword $verb $fact";
       $fact .= "." if($fact =~ /\w$/);
    } else {
       $fact = "$keyword $verb $fact";
       $fact .= "." if($fact =~ /\w$/);
    }


   if(length($fact) > $config{fact_length}) {
      $core->isInfo;
      return replace(gettext('Your fact is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($fact), MAX => $config{fact_length} });
   }
   
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';

    $core->log('Learn(): ' . replace(gettext('$KEYWORD saved/modified by $WHO'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}}), NORMAL);
    dbi_save_from_hash($dbh, $config{table}, 'keyword', { keyword => $keyword, fact => $fact });

    return;
}

sub LearnIndividual {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);
    my($keyword, $fact, $verb) = @$backrefs[0 .. 2];

    $keyword = lc($keyword);
    $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
    $keyword = $1;

   my $lengde = length($keyword);

   my %config;
   my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
   $core->module_config(\%config, $where);

   if($lengde > $config{keyword_length}) {
      $core->isInfo;
      return replace(gettext('Your keyword is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => $lengde, MAX => $config{keyword_length} });
   }

   #return replace(gettext('"$KEYWORD" is a stopword.'), { KEYWORD => $keyword }) if($stopwords{$keyword});

    #This implies that adding to a pointer really adds it to the original.
    my $oldfact; 
    ($keyword, $oldfact) = get_fact($core, $keyword);

    if ($oldfact) {
       #Append the new fact if one already exists.

       #Remove any whitespace or punctation and add new ones.
       $fact = "$oldfact->{fact} $fact";
       $fact .= "." if($fact =~ /\w$/);
    } else {
       $fact = "$fact";
       $fact .= "." if($fact =~ /\w$/);
    }


   if(length($fact) > $config{fact_length}) {
      $core->isInfo;
      return replace(gettext('Your fact is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($fact), MAX => $config{fact_length} });
   }
   
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';

    $core->log('Learn(): ' . replace(gettext('$KEYWORD saved/modified by $WHO'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}}), NORMAL);

    $fact =~ s/.*?=(.*)/$1/;
    dbi_save_from_hash($dbh, $config{table}, 'keyword', { keyword => $keyword, fact => $fact });

    return;
}


sub Forget {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);
    my($keyword, $fact) = @$backrefs[0, 1];

    $keyword = lc($keyword);
   $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;

   my %config;
   my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
   $core->module_config(\%config, $where);
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';


    $core->log('Forget(): ' . replace(gettext('$KEYWORD deleted by $WHO'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}}), NORMAL);
    prepare_and_exec($dbh, "DELETE FROM $config{table} WHERE keyword=? OR pointer=?", $keyword, $keyword);
    return;
}

sub Substitute {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);

    my($keyword, $delimiter, $string, $replacement, $flags) = @$backrefs[0 .. 4];

    $keyword = lc($keyword);
   $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;

    my $fact;
    ($keyword, $fact) = get_fact($core, $keyword);

    return unless($fact->{fact});

    #Do some fancy-schmanzy substitutions.
    $string      = quotemeta($string);
    $replacement = quotemeta($replacement);

    if($flags && $flags !~ /[^ig]/) {
      if($flags eq "i") {
        $fact->{fact} =~ s/$string/$replacement/i;
      } elsif ($flags eq "g") {
        $fact->{fact} =~ s/$string/$replacement/g;
      } else {
        $fact->{fact} =~ s/$string/$replacement/gi;
      }
    } else {
      $fact->{fact} =~ s/$string/$replacement/;
    }

    # Uhm. Dirty, but hey - it works. (Cleaning up after quotemeta)
    $fact->{fact} =~ s/\\\\/BACKSLASH/g;
    $fact->{fact} =~ s/\\//g;
    $fact->{fact} =~ s/BACKSLASH/\\/g;
    
    $core->log('Substitute(): ' . replace(gettext('$KEYWORD modified by $WHO'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}}), NORMAL);

   my %config;
   my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
   $core->module_config(\%config, $where);
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';

    dbi_save_from_hash($dbh, $config{table}, 'keyword', { keyword => $keyword, fact => $fact->{fact} });

    return;
}

sub Overwrite {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);

    my($keyword, $verb,  $fact) = @$backrefs[0 .. 2];

    $keyword = lc($keyword);
   $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;
    #return replace(gettext('"$KEYWORD" is a stopword.'), { KEYWORD => $keyword }) if($stopwords{$keyword});
    $core->heap->{keyword} = $keyword;

    my %config;
    my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
    $core->module_config(\%config, $where);
    
    if(length($keyword) > $config{keyword_length}) {
      $core->isInfo;
      return replace(gettext('Your keyword is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($keyword), MAX => $config{keyword_length} });
    }

    $fact = "$keyword $verb $fact";
    $fact .= "." if($fact =~ /\w$/);

    if(length($fact) > $config{fact_length}) {
      $core->isInfo;
      return replace(gettext('Your fact is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($fact), MAX => $config{fact_length} });
    }
 
    $core->log('Overwrite(): ' . replace(gettext('$KEYWORD overwritten by $WHO'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}}), NORMAL);

    my $dbh = fact_dbh($core, $where);

    $config{table} ||= 'facts';

    dbi_save_from_hash($dbh, $config{table}, 'keyword', { keyword => $keyword, fact => $fact });
    return;
}

#Tell by privmsg.
sub Tell {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);
    my($whom, $keyword ) = @$backrefs[0,1];

    return unless($keyword);
    $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;

    my $fact;
    ($keyword, $fact) = get_fact($core, $keyword);

    return unless($fact->{fact});

    $core->heap->{where} = $whom;
    $core->heap->{keyword} = $keyword;

    $core->log('Tell(): ' . replace(gettext('$WHO told $WHOM about "$KEYWORD"'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}, WHOM => $whom }), INFO);
    
    return($fact->{fact});
}

sub Pointer {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);
    my($keyword, $pointer ) = @$backrefs[0,1];

    my %config;
    my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
    $core->module_config(\%config, $where);

    if(length($keyword) > $config{keyword_length}) {
      $core->isInfo;
      return replace(gettext('Your keyword is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($keyword), MAX => $config{keyword_length} });
    }
    #return replace(gettext('"$KEYWORD" is a stopword.'), { KEYWORD => $keyword }) if($stopwords{$keyword});
    
    $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;
    # a -> b
    $keyword = lc($keyword); # a
    $pointer = lc($pointer); # b
    
   my %config;
   my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
   $core->module_config(\%config, $where);
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';


    #If a fact with the keyword "a" exists, it takes precedence
    my %fact = dbi_exec_for_row_hash( $dbh, "SELECT * FROM $config{table} WHERE keyword=?", $keyword);
    return if($fact{fact});
    %fact = undef;

    #Overwrite it if it's a pointer.

    #If a -> b and b -> c, then a -> c:
    my $fact;
    ($pointer, $fact) = get_fact($core, $pointer);
    return unless($fact->{fact});
    
    $core->heap->{keyword} = $keyword;

    $core->log('Pointer(): ' . replace(gettext('$WHO added new pointer: $KEYWORD -> $POINTER'),
       { KEYWORD => $keyword, WHO => $core->heap->{who}, POINTER => $pointer }), NORMAL);

    dbi_save_from_hash($dbh, $config{table}, 'keyword', { keyword => $keyword, pointer => $pointer });

    return;
}

#This grabs /* keyword */ out of a sentence, and stores the sentence without the /**/-s.
sub LearnMore {
    my($core, $backrefs, $args) = @_;
    return if($core->testing);
    my $sentence = $backrefs->[0];

    $sentence =~ m/.*?\/\* (.+?) \*\/.*?/;
    
    my $keyword = lc($1);
    return unless($keyword); #Shouldn't happen.
    $keyword =~ /^(?:\s*)(.*?)(?:\s*)$/; #weed out whitespace.
   $keyword = $1;

    #return replace(gettext('"$KEYWORD" is a stopword.'), { KEYWORD => $keyword }) if($stopwords{$keyword});
    $core->heap->{keyword} = $keyword;

    $sentence =~ s/\/\* //g;
    $sentence =~ s/ \*\///g;

    my $nicklen = $core->serverinfo->{NICKLEN};
    if($sentence =~ /^[^\s\$]{1,$nicklen}[:;,]{1}/) {
      $sentence =~ /^[^\s\$]{1,$nicklen}[:;,]{1}(.*)/;
      $sentence = $1;
    }

    #Remove leading whitespace.
    $sentence =~ /^(\s*)(.*)$/;
    $sentence = $2;
    $sentence .= "." if($sentence =~ /\w$/);

    my %config;
    my $where = $core->heap->{privmsg} ? 'privmsg' : $core->heap->{where};
    $core->module_config(\%config, $where);
   
    if(length($keyword) > $config{keyword_length}) {
      $core->isInfo;
      return replace(gettext('Your keyword is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($keyword), MAX => $config{keyword_length} });
    }
    if(length($sentence) > $config{fact_length}) {
      $core->isInfo;
      return replace(gettext('Your fact is way too long ($X characters). Please limit yourself to $MAX characters.'),
         { X => length($sentence), MAX => $config{fact_length} });
    }
 
   my $dbh = fact_dbh($core, $where);

   $config{table} ||= 'facts';

    #This will overwrite any existing fact, but that's cool. :)
    $core->log('LearnMore(): ' . replace(gettext('$WHO added new fact on $KEYWORD'), { KEYWORD => $keyword, WHO => $core->heap->{who} }), NORMAL);
    dbi_save_from_hash($dbh, $config{table}, 'keyword', { keyword => $keyword, fact => $sentence });
    return;
}

#Different channels and databases may be used, so provide an interface
#to use the different database handlers.
sub fact_dbh
{
   my ($core, $where) = @_;

   my %config;
   $core->module_config(\%config, $where);

   $config{name} = join " ", map { $config{$_} } qw(driver database username);

   if($core->dbh($config{name})) {
      return $core->dbh($config{name});
   } else {
      $core->dbi_connect(\%config);
      return $core->dbh($config{name});
   }
}


# Establish database connections.
# Different channels may use different databases.
# So based on where, connect to different databases and store them
# with different names. fact_dbh() is used to retrieve the different
# dbh-s.
sub init
{ 
   my $core = shift;

   my %config = %{ $core->config->{Modules}->{Facts} }; 

   # $core->log(gettext('Loading stopwords.'), INFO);
   # untie %stopwords;
   # tie %stopwords, 'GDBM_File', "var/stopwords", GDBM_WRCREAT, 0640 or
   #    $core->log(gettext('Couldn\'t load stopwords!'),  ERROR);

   foreach my $somewhere (grep(/((^[#&])|(^privmsg$))/, keys %config)) {
      my %someconfig = %{ $config{$somewhere} };

      foreach my $key (qw(driver database username password)) {
         $someconfig{$key} ||= $config{$key};
      }

      my $name = join " ", map { $someconfig{$_} } qw(driver database username);

      if($core->dbh($name)) {
         next;
      } else {
         $someconfig{name} = $name;
         $core->dbi_connect(\%someconfig);
         if($someconfig{'driver'} eq "SQLite") {
             setup_database($core, $name, \%config);
         }
      }
   }

   my $name = join " ", map { $config{$_} } qw(driver database username);
   $config{name} = $name;
   $core->dbi_connect(\%config);
   if($config{'driver'} eq "SQLite") {
       setup_database($core, $name, \%config);
   }
}

sub setup_database {
    my ($core, $name, $config) = @_;
    my $dbh = $core->dbh($name);
    my $table = $config->{table} || 'facts';

    if(! dbi_exec_for_scalar($dbh, "select exists(select 1 from sqlite_master where tbl_name='$table')")) {
        prepare_and_exec($dbh, 'CREATE TABLE facts(keyword, pointer, fact)');
    }
}

1;
