package Paste;

# $Id$
#
# Pastebin functionality.
# Available configuration options:
#   template => Template to use. See 'template.html' for an example. (Yes, the template functionality is butt ugly)
#   path => Path to directory where the HTML-files will be stored.
#   url_prefix => url_prefix for generated urls. "/pasteid.html" will be appended.
#   expire => expire the pasting session after this many seconds if no input is retrieved. Default: 120 seconds.

use base qw(Plugin);
use strict;
use Common;

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
bindtextdomain('erwin', '/usr/share/locale/');
textdomain('erwin');

my $pastedir = 'paste';

sub PasteInit
{
   my ( $core, $backrefs, $args ) = @_;
   return if($core->testing || $core->heap->{public});
   my $title = $backrefs->[0];

   $core->isInfo;

   return gettext('Erroneous title. Title must be more than 4 and less than 200 characters and not containt a question. It\'s a descriptive title, not a question!') unless(length($title) > 4 && length($title) < 200 && $title !~ /\?/);

   #I may run paste bins for multiple channels in which the users resides.
   #Find out if the user is in more than one of those channels, and prompt him which
   #one he wants to paste in.

   #Channels the user resides in.
   my @userchannels = $core->getUserChannels($core->heap->{nick});

   unless(@userchannels) {
      $core->msg($core->heap->{nick}, gettext('You\'re not in any channels in which I linger.'));
      return;
   }

   my %config = %{ $core->config->{Modules}->{Paste} }; #Paste configuration, with hash of channels.

   my (@isect, %isect);
   foreach my $channel (@userchannels) {
      $isect{$channel} = 1 if($config{$channel});
   }

   @isect = keys %isect;

   my $paste = { title => $title, lines => [] };

   if($#isect > 0 ) {
      $core->msg($core->heap->{nick}, gettext('I have pasting enabled for multiple channels in which we both linger. Please select which one you want to paste in.'));

      my $x;
      $core->msg($core->heap->{nick},
         replace( gettext('Enter the number of the channel you would like to paste in: $LIST'), { LIST => join ", ", map { $x++; "$x: $_ "; } @isect }) );

   } elsif($#isect == 0) {
      #He's only in one of the channels in which I may run paste bins.
      $paste->{channel} = $isect[0];
   
      instructions($core);
   } else {
      $core->msg($core->heap->{nick}, gettext('Pasting is not enabled in any of the channels we\'re both in.'));
      return;
   }

   #Take control over the privmsg events.
   $core->plugin->{privmsg}->{$core->heap->{who}} = 'Paste';

   #Store the paste session.
   $config{expire} ||= 120;
   $core->var({ namespace => 'paste', key => $core->heap->{who}, expires => $config{expire}, value => $paste });

   return;
}

sub instructions
{
   my $core = shift;
   $core->msg($core->heap->{nick}, gettext('You now have about two minutes to paste the content, after which it will expire. Terminate your paste by typing "!pasted" on a line of its own.'));
   $core->msg($core->heap->{nick}, gettext('If you should want to abort, type "!abort" on a line of its own.'));
}

sub privmsg
{
   my ($core,$who,$msg) = @_;

   return unless($core->plugin->{privmsg}->{$who});

   my $paste = $core->var({ key => $who, namespace => 'paste' });

   unless($paste->{channel}) {
      if($msg =~ /\D/) {
         $core->msg($core->heap->{nick},gettext("Invalid input. Please enter a number identifying the channel."));
	 return;
      } else {
         my %config = %{ $core->config->{Modules}->{Paste} }; #Paste configuration, with hash of channels.
         my (@isect, %isect);

         my @userchannels = $core->getUserChannels($core->heap->{nick});

         foreach my $channel (@userchannels) {
            $isect{$channel} = 1 if($config{$channel});
         }
      
         @isect = keys %isect;

	 if($isect[$msg - 1]) {
	    $paste->{channel} = $isect[$msg - 1];
            instructions($core);
	    $config{expire} ||= 120;
            $core->var({ key => $who, namespace => 'paste', expires => $config{expire}, value => $paste });
	 } else {
            $core->msg($core->heap->{nick},gettext("Invalid input. Please enter a number identifying the channel."));
	 }

	 return;
      }
   }

   my %config;
   $core->module_config(\%config, $paste->{channel});

   my $pasted = gettext('^!pasted');
   my $abort = gettext('^!abort');


   if($msg =~ /$pasted/i) {
      # !pasted is typed, so do something useful with the pasted lines.

      #release control
      delete $core->plugin->{privmsg}->{$who};
      $core->var({ delete => $who, namespace => 'paste' });

      if(@{$paste->{lines}}) {

         my $pasteid = $core->upcounter({ key => 'pasteid', namespace=>'paste', expires => -1 });
         $paste->{who} = $who;
         $paste->{template} = $config{template};
	 my $file = "$config{path}/$pasteid.html";
         open(PASTE, ">$file") or $core->log(replace(gettext('Couldn\'t open $FILE for writing!'), { FILE => $file}), ERROR) &&
	    $core->msg($core->heap->{nick},replace(gettext('An error occured: $ERROR. Contact admin.'), { ERROR => $1 }));

         print PASTE MakeHTML($paste);
         close(PASTE);
         
         my $url = "$config{url_prefix}/$pasteid.html";

         $core->msg($core->heap->{nick},replace(gettext('Okay. Publishing URL on "$CHANNEL"'),
            { CHANNEL => $paste->{channel}}));

         $core->msg($paste->{channel},replace(gettext('$WHO has pasted "$TITLE". $URL'),
            { WHO => $core->heap->{nick}, TITLE=>$paste->{title}, URL => $url}));
      } else {
         $core->msg($core->heap->{nick},gettext('Nothing pasted. Aborting.'));
	 $core->var({ delete => $who, namespace => 'paste' });
      }

      return;
   } elsif($msg =~ /$abort/) {
      #abort and release control.
      $core->var({ delete => $who, namespace => 'paste' });
      $core->msg('erwin','privmsg',$core->heap->{nick},gettext('Paste aborted.'));
      delete $core->plugin->{privmsg}->{$who};
      return;
   } else {
      #A pasted line.
      push(@{$paste->{lines}}, $msg);
      $config{expire} ||= 120;
      $core->var({ key => $who, namespace => 'paste', expires => $config{expire}, value => $paste });
      return;
   }

}

#Warning: INSANE template function ahead. You're warned ;-)

sub MakeHTML
{
   my $arg = shift;

   for(qw(template who lines title)) {
      return unless $arg->{$_};
   }

   open(FILE, $arg->{template}) or die;
   my $template = join('', <FILE>);
   close(FILE);

   $template =~ /<% line %>(.*)<% \/line %>/i;
   my $linetemplate = $1;

   my @content;

   my $x = 1;
   foreach my $line (@{$arg->{lines}}) {
      chomp(my $_line = $linetemplate);
      $line =~ s/</&lt;/g;
      $line =~ s/>/&gt;/g;
      $_line =~ s/<% content %>/$line/gi;
      $_line =~ s/<% X %>/$x/gi;
      push(@content, $_line);
      $x++;
   }

   my $content = join "\n", @content;

   @content = ();
   foreach my $line (split /\n/, $template) {
      $line =~ s/<% line %>.*<% \/line %>/$content/i;
      $line =~ s/<% title %>/$arg->{title}/gi;
      $line =~ s/<% who %>/$arg->{who}/gi;
      push(@content, "$line\n");
   }

   return @content;
}

__PACKAGE__->install({
	events => { privmsg => 1 },
});
