package Users;

# $Id$
# Contains functions for modifying users.

use Digest::MD5 qw(md5_hex);
use Common;
use User;

use base qw(Plugin);

use My::DBI qw(:all);

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
bindtextdomain('erwin', '/usr/share/locale/');
textdomain('erwin');

use strict;
use warnings;

sub Password {
    my ( $core, $backrefs, $args ) = @_;
    return if($core->testing);
    my ($oldpassword, $newpassword ) = @$backrefs[0,1];

    my $user = User->GetByHostmask($core->dbh('users'), $core->heap->{who});

    return gettext('You\'re not logged in.') unless($user);
    return gettext('Old password incorrect.') unless(md5_hex($oldpassword) eq $user->password);

    $user->password($newpassword);
    $user->save;

    return gettext('Password changed. Please remember it.');
}

sub Auth {
   my ( $core, $backrefs, $args ) = @_;
   return if($core->testing);
   my ($username, $password ) = split(/ /, $backrefs->[0], 2);

   if($username && $password) {
      my $User = User->GetByName($core->dbh('users'), $username);
      return gettext('Invalid credentials.') unless($User && md5_hex($password) eq $User->password);

      $User->setHostmask($core->heap->{who});
      $core->isInfo;
      return gettext('Logged in.');
   } elsif($username) {
      my $User = User->GetByName($core->dbh('users'), $username);
      if($User) {
         $core->startConversation($core->heap->{nick}, 'login_password', gettext('Please enter your password.'), { user => $User });
      } else {
	 return gettext('No user with that name exists.');
      }
   } else {
       $core->startConversation($core->heap->{nick}, 'login_username', gettext('Please enter your username.'));
   }
}

sub login_username {
   my ($core, $data, $msg) = @_;

   $msg =~ s/\s//g;

   if($msg =~ /\W/) {
      $core->cmsg(gettext('Invalid characters in username.'));
	 return undef;
   }

   my $User = User->GetByName($core->dbh('users'), $msg);
   if($User) {
      $data->{user} = $User;
      $core->cmsg(gettext('Please enter your password.'));
      return 'login_password';
   } else {
      $core->cmsg(gettext('No user with that name exists.'));
      return undef; #End of conversation.
   }
}

sub login_password {
   my ($core, $data, $msg) = @_;

   my $User = $data->{user};

   if($User->password eq md5_hex($msg)) {
      $User->setHostmask($core->heap->{who});
      $core->cmsg(gettext('Logged in.'));
   } else {
      $core->cmsg(gettext('Invalid credentials.'));
   }

   return undef; #End of conversation.
}

sub Logout {
    my ( $core, $backrefs, $args ) = @_;
    return if($core->testing);

    my $user = User->GetByHostmask($core->dbh('users'), $core->heap->{who});

    if($user) {
       $user->hostmask('-');
       $user->save;
       $core->isInfo;
       return gettext('Logged out.');
    } else {
       return gettext('Not logged in.');
    }
}


sub Register {
    my ( $core, $backrefs, $args ) = @_;
    return if($core->testing);
    my ($username, $password ) = @$backrefs[0,1];

    $username = lc($username);

    return gettext('Registration is not open.') if(!$core->config->{Users}->{Public});

    my $dbh = $core->dbh('users');

    return replace(gettext('A user with username "$USERNAME" already exists.'), { USERNAME => $username }) if (User->GetByName($dbh, $username));

    my $flags = '';
    if(! dbi_exec_for_scalar($dbh, 'select exists(select 1 from users)')) {
        $flags = "a";
    }

    #Save the user.
    dbi_insert_from_hash($dbh, 'users', { username => $username, flags => $flags, active => 1, password => md5_hex($password), hostmask => $core->heap->{who} });

    $core->isInfo;
    return replace(gettext('User "$USERNAME" created, flags "$FLAGS". Please remember your password.'), { USERNAME => $username, FLAGS => $flags });

}

sub Adduser {
    my ( $core, $backrefs, $args ) = @_;
    return if($core->testing);
    my ($username, $password ) = @$backrefs[0,1];

    $username = lc($username);

    my $dbh = $core->dbh('users');

    return replace(gettext('A user with username "$USERNAME" already exists.'), { USERNAME => $username }) if (User->GetByName($dbh, $username));

    #Save the user.
    dbi_insert_from_hash($dbh, 'users', { username => $username, flags => '', active => 1, password => md5_hex($password) });

    $core->isInfo;
    return replace(gettext('User "$USERNAME" created.'), { USERNAME => $username });
}

sub Delete {
    my ( $core, $backrefs, $args ) = @_;
    return if($core->testing);
    my $username = lc($backrefs->[0]);

    $core->isInfo;

    my $dbh = $core->dbh('users');

    return gettext('Non-existant username.') unless(User->GetByName($dbh, $username));
    #Don't delete yourself.
#    return gettext('I\'m not letting you commit suicide.') if(User->GetByHostname($dbh, $core->heap->{who})->username eq $username);


    prepare_and_exec($dbh, 'DELETE FROM users WHERE lower(username)=?', lc($username));

    return replace(gettext('User "$USERNAME" deleted.'), { USERNAME => $username }); 

}

sub Grant {
    my ( $core, $backrefs, $args ) = @_;
    return if($core->testing);
    my ($username, $flags) = @$backrefs[0,1];

    $core->isInfo;
    return gettext('You can\'t edit your own flags.') if($core->users->{$core->heap->{who}} eq $username);

    my $dbh = $core->dbh('users');
    my $user = User->GetByName($dbh, $username);
    return gettext('Non-existant username.') unless($user);

    $user->flags($flags);
    $user->save;

    return gettext('Flags updated.');

}

sub nick {
   my ($core, $who, $newnick) = @_;

   my $User = User->GetByHostmask($core->dbh('users'), $who);

   $who =~ /!(.*)/;
   my $hostmask = "$newnick!$1";

   $User->setHostmask($hostmask);

}

sub first_run {
   my $core = shift;
   my %config;   
   $core->module_config(\%config);

   if($config{database}) {
      $core->dbi_connect({ database => $config{database}, username => $config{username}, password => $config{password}, driver => $config{driver}, name => 'users' });
   } else {
      $core->dbh('users', $core->dbh('erwin')); #Set to be the same.
   }
}

__PACKAGE__->install({ events => { nick => 1 } });

1;
