#Abstract class for Erwin plugins.
#$Id$

=head1 NAME

Abstract class for Erwin plugins.

=cut

=head1 SYNOPSIS

use base qw(Plugin);

=cut

=head1 DESCRIPTION

Inherit from this class to write plugins.

For every plugin installed, Erwin will run a subroutine for their respective events.

=head2 Events

=over 2

=cut


package Plugin;

use strict;
use warnings;

#use vars qw(@ISA @EXPORT @EXPORT_OK);

#require Exporter;

#@ISA = qw(Exporter);
#@EXPORT = qw(privmsg public install);
#@EXPORT_OK = qw();

=item B<privmsg($core, $who, $msg)>

$who sent $msg to me in private.

=cut

sub privmsg {}

=item B<public($core, $who, $where, $msg)>

$who sent $msg to $where.

=cut

sub public {}

=item B<on_connect($core)>

The bot has just connected to a server.

=cut

sub on_connect {}

=item B<disconnected($core)>

The bot got disconnected from a server.

=cut

sub disconnected {}

=item B<stop($core)>

POE session is about to stop.

=cut

sub stop {}

=item B<join($core, $who, $where)>

$who joined $where.

=cut

sub join {}

=item B<self_join($core, $where)>

I joined $where.

=cut

sub self_join {}

=item B<who($core, $string)>

A response to /WHO is recieved.

$string is the raw response from the IRC server.

=cut

sub who {}

=item B<mode($core, $bywhom,$where,$mode,@args)>

A mode change on $where by $bywhom.

$mode is the mode string. @args are the operands to the mode string.

=cut

sub mode {}

=item B<kick($core, $kicker,$where,$kicked, $kickmsg)>

$kicked kicked $kicked out of $where with $kickmsg as explanation.

=cut

sub kick {}

=item B<quit($core, $who, $msg)>

$who quit IRC with $msg as the quit message.

=cut

sub quit {}

=item B<part($core, $who, $where)>

$who left $where.

=cut

sub part {}

=item B<nick($core, $who, $newnick)>

$who changed nick to $newnick.

=cut

sub nick {}

=item B<first_run($core)>

Run the first time the module is loaded, after $core is initialized.

=cut

sub first_run {}

=item B<cron($core)>

Run every now and then. See the "cron_interval" configuration variable.

=cut

=item __PACKAGE__->B<install>(\%args)

Configures Erwin to use the plugin. 

This should be the only thing run in the plugin.

Arguments:

B<immutable> -- Boolean. The plugin recieves events even though another plugin has taken control. (Useful for e.g. statistic collectors)

B<events> -- Hashref of events the module should recieve. Possible keys are privmsg, public, on_connect, disconnected, stop, join, self_join, who, mode, kick, quit, part, nick, first_run and cron.


=cut

sub install
{
   my $plugin = shift;
   my $args = shift;

   $args ||= {};

   $Core::plugins{$plugin} = $args;

}

1;
