package User;
use warnings;
use strict;

use vars qw($dbh);

use My::DBI qw(:all);
use Digest::MD5 qw(md5_hex);

sub new
{
   my($self, $dbh, $username) = @_;

   if(! dbi_exists($dbh, "SELECT username FROM users WHERE username=?", $username) ) {
      return bless { username => lc($username), dbh => $dbh, active => 't' }, $self;
   } else {
      return 0;
   }
}

sub GetByName
{
   my ($self, $dbh, $username) = @_;

   my %user = dbi_exec_for_row_hash($dbh, 'SELECT * FROM users WHERE LOWER(username)=?', lc($username));

   return undef unless keys %user;
   $user{dbh} = $dbh;

   return bless \%user, $self;
}

=cut
sub GetByID
{
   my ($self, $dbh, $id) = @_;

   my %user = dbi_exec_for_row_hash($dbh, "SELECT * FROM users WHERE id=?", $id);
   $user{dbh} = $dbh;

   return bless \%user, $self;
}
=cut

sub GetByHostmask
{
   my ($self, $dbh, $hostmask) = @_;

=cut
   if($hostmask =~ /!/) {
      $hostmask =~ /(.*)!(.*)/;
      $hostmask = $2;
   }
=cut
   my %user = dbi_exec_for_row_hash($dbh, 'SELECT * FROM users WHERE LOWER(hostmask)=?', lc($hostmask));

   return undef unless keys %user;
   $user{dbh} = $dbh;

   return bless \%user, $self;
}

sub setHostmask
{
   my $self = shift;
   my $hostmask = shift;

=cut
   if($hostmask =~ /!/) {
      $hostmask =~ /.*!(.*)/;
      $hostmask = $1;
   }
=cut
   
   prepare_and_exec($self->dbh, 'UPDATE users SET hostmask=null WHERE hostmask=?', $hostmask);
   prepare_and_exec($self->dbh, 'UPDATE users SET hostmask=? WHERE username=?', $hostmask, $self->username);
}

sub save
{
   my $self = shift;

   my %row = map { $_ => $self->{$_} } qw(username hostmask flags password active);
   dbi_save_from_hash($self->dbh, 'users', 'username', \%row); 
}

sub delete
{
   my $self = shift;

   prepare_and_exec($self->dbh, "DELETE FROM users WHERE username=?", $self->username);
}

sub nick
{
   my $self = shift;

   $self->{hostmask} =~ /(.*)!/;
   return $1;

}

sub password
{
   my $self = shift;
   my $password = shift;

   if($password) {
      $self->{password} = md5_hex($password);
   }
   return $self->{password};
}

#cannot be set.
sub id { return shift->{id}; }

sub AUTOLOAD {
    my $self = shift;
    my $attr = our $AUTOLOAD;
    $attr =~ s/.*:://;
    return unless $attr =~ /[^A-Z]/;  # skip DESTROY and all-cap methods

    my %ok_field = map { $_ => 1 } qw(username hostmask flags password active dbh);
    die "invalid attribute method: ->$attr( )" unless $ok_field{$attr};

    $self->{lc $attr} = shift if @_;
    return $self->{lc $attr};
}

1;
